/*
  Q Light Controller
  artnetout.h
  
  Copyright (c) Heikki Junnila
                Simon Newton
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  Version 2 as published by the Free Software Foundation.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details. The license is
  in the file "COPYING".
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef ARTNETOUT_H
#define ARTNETOUT_H

#include "../common/outputplugin.h"
#include "../../main/src/types.h"
#include "artnetthread.h"

#include <qptrlist.h>
#include <qstring.h>

#define CHANNELS 512

class ConfigureArtNetOut;
class QPoint;

extern "C" OutputPlugin* create(t_plugin_id id);
extern "C" void destroy(OutputPlugin* object);

class ArtNetOut : public OutputPlugin
{
  Q_OBJECT

  friend class ConfigureArtNetOut;

 public:
  ArtNetOut(t_plugin_id id);
  ~ArtNetOut();

  // Plugin methods
  int open();
  int close();
  bool isOpen();
  int configure();
  QString infoText();
  void contextMenu(QPoint pos);

  int setConfigDirectory(QString dir);
  int saveSettings();
  int loadSettings();

  // OutputPlugin methods
  int writeChannel(t_channel channel, t_value value);
  int writeRange(t_channel address, t_value* values, t_channel num);

  int readChannel(t_channel channel, t_value &value);
  int readRange(t_channel address, t_value* values, t_channel num);

 private slots:
  void slotContextMenuCallback(int item);

 private:
  void activate();
  void createContents(QPtrList <QString> &list);

 private:
  QString m_configDir;			// config dir
  t_value m_values[KChannelMax];	//channel data
  ArtNetThread *m_thr ;
  QString m_ip;  
};

#endif
